# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import meta_v1


@dataclass
class CrossVersionObjectReference(DictMixin):
    """CrossVersionObjectReference contains enough information to let you identify
      the referred resource.

      **parameters**

      * **kind** ``str`` - Kind of the referent; More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **name** ``str`` - Name of the referent; More info:
        http://kubernetes.io/docs/user-guide/identifiers#names
      * **apiVersion** ``Optional[str]`` - API version of the referent
    """
    kind: 'str'
    name: 'str'
    apiVersion: 'Optional[str]' = None


@dataclass
class HorizontalPodAutoscaler(DictMixin):
    """configuration of a horizontal pod autoscaler.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[HorizontalPodAutoscalerSpec]`` - behaviour of autoscaler. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.
      * **status** ``Optional[HorizontalPodAutoscalerStatus]`` - current information about the autoscaler.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[HorizontalPodAutoscalerSpec]' = None
    status: 'Optional[HorizontalPodAutoscalerStatus]' = None


@dataclass
class HorizontalPodAutoscalerList(DictMixin):
    """list of horizontal pod autoscaler objects.

      **parameters**

      * **items** ``List[HorizontalPodAutoscaler]`` - list of horizontal pod autoscaler objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata.
    """
    items: 'List[HorizontalPodAutoscaler]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class HorizontalPodAutoscalerSpec(DictMixin):
    """specification of a horizontal pod autoscaler.

      **parameters**

      * **maxReplicas** ``int`` - upper limit for the number of pods that can be set by the autoscaler; cannot
        be smaller than MinReplicas.
      * **scaleTargetRef** ``CrossVersionObjectReference`` - reference to scaled resource; horizontal pod autoscaler will learn the current
        resource consumption and will set the desired number of pods by using its
        Scale subresource.
      * **minReplicas** ``Optional[int]`` - minReplicas is the lower limit for the number of replicas to which the
        autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to
        be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one
        Object or External metric is configured.  Scaling is active as long as at
        least one metric value is available.
      * **targetCPUUtilizationPercentage** ``Optional[int]`` - target average CPU utilization (represented as a percentage of requested CPU)
        over all the pods; if not specified the default autoscaling policy will be
        used.
    """
    maxReplicas: 'int'
    scaleTargetRef: 'CrossVersionObjectReference'
    minReplicas: 'Optional[int]' = None
    targetCPUUtilizationPercentage: 'Optional[int]' = None


@dataclass
class HorizontalPodAutoscalerStatus(DictMixin):
    """current status of a horizontal pod autoscaler

      **parameters**

      * **currentReplicas** ``int`` - current number of replicas of pods managed by this autoscaler.
      * **desiredReplicas** ``int`` - desired number of replicas of pods managed by this autoscaler.
      * **currentCPUUtilizationPercentage** ``Optional[int]`` - current average CPU utilization over all pods, represented as a percentage of
        requested CPU, e.g. 70 means that an average pod is using now 70% of its
        requested CPU.
      * **lastScaleTime** ``Optional[meta_v1.Time]`` - last time the HorizontalPodAutoscaler scaled the number of pods; used by the
        autoscaler to control how often the number of pods is changed.
      * **observedGeneration** ``Optional[int]`` - most recent generation observed by this autoscaler.
    """
    currentReplicas: 'int'
    desiredReplicas: 'int'
    currentCPUUtilizationPercentage: 'Optional[int]' = None
    lastScaleTime: 'Optional[meta_v1.Time]' = None
    observedGeneration: 'Optional[int]' = None


@dataclass
class Scale(DictMixin):
    """Scale represents a scaling request for a resource.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.
      * **spec** ``Optional[ScaleSpec]`` - defines the behavior of the scale. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.
      * **status** ``Optional[ScaleStatus]`` - current status of the scale. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.
        Read-only.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[ScaleSpec]' = None
    status: 'Optional[ScaleStatus]' = None


@dataclass
class ScaleSpec(DictMixin):
    """ScaleSpec describes the attributes of a scale subresource.

      **parameters**

      * **replicas** ``Optional[int]`` - desired number of instances for the scaled object.
    """
    replicas: 'Optional[int]' = None


@dataclass
class ScaleStatus(DictMixin):
    """ScaleStatus represents the current status of a scale subresource.

      **parameters**

      * **replicas** ``int`` - actual number of observed instances of the scaled object.
      * **selector** ``Optional[str]`` - label query over pods that should match the replicas count. This is same as
        the label selector but in the string format to avoid introspection by clients.
        The string will be in the same format as the query-param syntax. More info
        about label selectors:
        http://kubernetes.io/docs/user-guide/labels#label-selectors
    """
    replicas: 'int'
    selector: 'Optional[str]' = None





# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import core_v1
from . import meta_v1


@dataclass
class AllocationResult(DictMixin):
    """AllocationResult contains attributed of an allocated resource.

      **parameters**

      * **availableOnNodes** ``Optional[core_v1.NodeSelector]`` - This field will get set by the resource driver after it has allocated the
        resource driver to inform the scheduler where it can schedule Pods using the
        ResourceClaim.
        Setting this field is optional. If null, the resource is available everywhere.
      * **resourceHandle** ``Optional[str]`` - ResourceHandle contains arbitrary data returned by the driver after a
        successful allocation. This is opaque for Kubernetes. Driver documentation may
        explain to users how to interpret this data if needed.
        The maximum size of this field is 16KiB. This may get increased in the future,
        but not reduced.
      * **shareable** ``Optional[bool]`` - Shareable determines whether the resource supports more than one consumer at a
        time.
    """
    availableOnNodes: 'Optional[core_v1.NodeSelector]' = None
    resourceHandle: 'Optional[str]' = None
    shareable: 'Optional[bool]' = None


@dataclass
class PodScheduling(DictMixin):
    """PodScheduling objects hold information that is needed to schedule a Pod with
      ResourceClaims that use "WaitForFirstConsumer" allocation mode.
      
      This is an alpha type and requires enabling the DynamicResourceAllocation
      feature gate.

      **parameters**

      * **spec** ``PodSchedulingSpec`` - Spec describes where resources for the Pod are needed.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **status** ``Optional[PodSchedulingStatus]`` - Status describes where resources for the Pod can be allocated.
    """
    spec: 'PodSchedulingSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[PodSchedulingStatus]' = None


@dataclass
class PodSchedulingList(DictMixin):
    """PodSchedulingList is a collection of Pod scheduling objects.

      **parameters**

      * **items** ``List[PodScheduling]`` - Items is the list of PodScheduling objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[PodScheduling]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class PodSchedulingSpec(DictMixin):
    """PodSchedulingSpec describes where resources for the Pod are needed.

      **parameters**

      * **potentialNodes** ``Optional[List[str]]`` - PotentialNodes lists nodes where the Pod might be able to run.
        The size of this field is limited to 128. This is large enough for many
        clusters. Larger clusters may need more attempts to find a node that suits all
        pending resources. This may get increased in the future, but not reduced.
      * **selectedNode** ``Optional[str]`` - SelectedNode is the node for which allocation of ResourceClaims that are
        referenced by the Pod and that use "WaitForFirstConsumer" allocation is to be
        attempted.
    """
    potentialNodes: 'Optional[List[str]]' = None
    selectedNode: 'Optional[str]' = None


@dataclass
class PodSchedulingStatus(DictMixin):
    """PodSchedulingStatus describes where resources for the Pod can be allocated.

      **parameters**

      * **resourceClaims** ``Optional[List[ResourceClaimSchedulingStatus]]`` - ResourceClaims describes resource availability for each pod.spec.resourceClaim
        entry where the corresponding ResourceClaim uses "WaitForFirstConsumer"
        allocation mode.
    """
    resourceClaims: 'Optional[List[ResourceClaimSchedulingStatus]]' = None


@dataclass
class ResourceClaim(DictMixin):
    """ResourceClaim describes which resources are needed by a resource consumer. Its
      status tracks whether the resource has been allocated and what the resulting
      attributes are.
      
      This is an alpha type and requires enabling the DynamicResourceAllocation
      feature gate.

      **parameters**

      * **spec** ``ResourceClaimSpec`` - Spec describes the desired attributes of a resource that then needs to be
        allocated. It can only be set once when creating the ResourceClaim.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **status** ``Optional[ResourceClaimStatus]`` - Status describes whether the resource is available and with which attributes.
    """
    spec: 'ResourceClaimSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[ResourceClaimStatus]' = None


@dataclass
class ResourceClaimConsumerReference(DictMixin):
    """ResourceClaimConsumerReference contains enough information to let you locate
      the consumer of a ResourceClaim. The user must be a resource in the same
      namespace as the ResourceClaim.

      **parameters**

      * **name** ``str`` - Name is the name of resource being referenced.
      * **resource** ``str`` - Resource is the type of resource being referenced, for example "pods".
      * **uid** ``str`` - UID identifies exactly one incarnation of the resource.
      * **apiGroup** ``Optional[str]`` - APIGroup is the group for the resource being referenced. It is empty for the
        core API. This matches the group in the APIVersion that is used when creating
        the resources.
    """
    name: 'str'
    resource: 'str'
    uid: 'str'
    apiGroup: 'Optional[str]' = None


@dataclass
class ResourceClaimList(DictMixin):
    """ResourceClaimList is a collection of claims.

      **parameters**

      * **items** ``List[ResourceClaim]`` - Items is the list of resource claims.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClaim]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClaimParametersReference(DictMixin):
    """ResourceClaimParametersReference contains enough information to let you locate
      the parameters for a ResourceClaim. The object must be in the same namespace
      as the ResourceClaim.

      **parameters**

      * **kind** ``str`` - Kind is the type of resource being referenced. This is the same value as in
        the parameter object's metadata, for example "ConfigMap".
      * **name** ``str`` - Name is the name of resource being referenced.
      * **apiGroup** ``Optional[str]`` - APIGroup is the group for the resource being referenced. It is empty for the
        core API. This matches the group in the APIVersion that is used when creating
        the resources.
    """
    kind: 'str'
    name: 'str'
    apiGroup: 'Optional[str]' = None


@dataclass
class ResourceClaimSchedulingStatus(DictMixin):
    """ResourceClaimSchedulingStatus contains information about one particular
      ResourceClaim with "WaitForFirstConsumer" allocation mode.

      **parameters**

      * **name** ``Optional[str]`` - Name matches the pod.spec.resourceClaims[*].Name field.
      * **unsuitableNodes** ``Optional[List[str]]`` - UnsuitableNodes lists nodes that the ResourceClaim cannot be allocated for.
        The size of this field is limited to 128, the same as for
        PodSchedulingSpec.PotentialNodes. This may get increased in the future, but
        not reduced.
    """
    name: 'Optional[str]' = None
    unsuitableNodes: 'Optional[List[str]]' = None


@dataclass
class ResourceClaimSpec(DictMixin):
    """ResourceClaimSpec defines how a resource is to be allocated.

      **parameters**

      * **resourceClassName** ``str`` - ResourceClassName references the driver and additional parameters via the name
        of a ResourceClass that was created as part of the driver deployment.
      * **allocationMode** ``Optional[str]`` - Allocation can start immediately or when a Pod wants to use the resource.
        "WaitForFirstConsumer" is the default.
      * **parametersRef** ``Optional[ResourceClaimParametersReference]`` - ParametersRef references a separate object with arbitrary parameters that will
        be used by the driver when allocating a resource for the claim.
        The object must be in the same namespace as the ResourceClaim.
    """
    resourceClassName: 'str'
    allocationMode: 'Optional[str]' = None
    parametersRef: 'Optional[ResourceClaimParametersReference]' = None


@dataclass
class ResourceClaimStatus(DictMixin):
    """ResourceClaimStatus tracks whether the resource has been allocated and what
      the resulting attributes are.

      **parameters**

      * **allocation** ``Optional[AllocationResult]`` - Allocation is set by the resource driver once a resource has been allocated
        successfully. If this is not specified, the resource is not yet allocated.
      * **deallocationRequested** ``Optional[bool]`` - DeallocationRequested indicates that a ResourceClaim is to be deallocated.
        The driver then must deallocate this claim and reset the field together with
        clearing the Allocation field.
        While DeallocationRequested is set, no new consumers may be added to
        ReservedFor.
      * **driverName** ``Optional[str]`` - DriverName is a copy of the driver name from the ResourceClass at the time
        when allocation started.
      * **reservedFor** ``Optional[List[ResourceClaimConsumerReference]]`` - ReservedFor indicates which entities are currently allowed to use the claim. A
        Pod which references a ResourceClaim which is not reserved for that Pod will
        not be started.
        There can be at most 32 such reservations. This may get increased in the
        future, but not reduced.
    """
    allocation: 'Optional[AllocationResult]' = None
    deallocationRequested: 'Optional[bool]' = None
    driverName: 'Optional[str]' = None
    reservedFor: 'Optional[List[ResourceClaimConsumerReference]]' = None


@dataclass
class ResourceClaimTemplate(DictMixin):
    """ResourceClaimTemplate is used to produce ResourceClaim objects.

      **parameters**

      * **spec** ``ResourceClaimTemplateSpec`` - Describes the ResourceClaim that is to be generated.
        This field is immutable. A ResourceClaim will get created by the control plane
        for a Pod when needed and then not get updated anymore.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
    """
    spec: 'ResourceClaimTemplateSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class ResourceClaimTemplateList(DictMixin):
    """ResourceClaimTemplateList is a collection of claim templates.

      **parameters**

      * **items** ``List[ResourceClaimTemplate]`` - Items is the list of resource claim templates.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClaimTemplate]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClaimTemplateSpec(DictMixin):
    """ResourceClaimTemplateSpec contains the metadata and fields for a
      ResourceClaim.

      **parameters**

      * **spec** ``ResourceClaimSpec`` - Spec for the ResourceClaim. The entire content is copied unchanged into the
        ResourceClaim that gets created from this template. The same fields as in a
        ResourceClaim are also valid here.
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - ObjectMeta may contain labels and annotations that will be copied into the PVC
        when creating it. No other fields are allowed and will be rejected during
        validation.
    """
    spec: 'ResourceClaimSpec'
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class ResourceClass(DictMixin):
    """ResourceClass is used by administrators to influence how resources are
      allocated.
      
      This is an alpha type and requires enabling the DynamicResourceAllocation
      feature gate.

      **parameters**

      * **driverName** ``str`` - DriverName defines the name of the dynamic resource driver that is used for
        allocation of a ResourceClaim that uses this class.
        Resource drivers have a unique name in forward domain order
        (acme.example.com).
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata
      * **parametersRef** ``Optional[ResourceClassParametersReference]`` - ParametersRef references an arbitrary separate object that may hold parameters
        that will be used by the driver when allocating a resource that uses this
        class. A dynamic resource driver can distinguish between parameters stored
        here and and those stored in ResourceClaimSpec.
      * **suitableNodes** ``Optional[core_v1.NodeSelector]`` - Only nodes matching the selector will be considered by the scheduler when
        trying to find a Node that fits a Pod when that Pod uses a ResourceClaim that
        has not been allocated yet.
        Setting this field is optional. If null, all nodes are candidates.
    """
    driverName: 'str'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    parametersRef: 'Optional[ResourceClassParametersReference]' = None
    suitableNodes: 'Optional[core_v1.NodeSelector]' = None


@dataclass
class ResourceClassList(DictMixin):
    """ResourceClassList is a collection of classes.

      **parameters**

      * **items** ``List[ResourceClass]`` - Items is the list of resource classes.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata
    """
    items: 'List[ResourceClass]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ResourceClassParametersReference(DictMixin):
    """ResourceClassParametersReference contains enough information to let you locate
      the parameters for a ResourceClass.

      **parameters**

      * **kind** ``str`` - Kind is the type of resource being referenced. This is the same value as in
        the parameter object's metadata.
      * **name** ``str`` - Name is the name of resource being referenced.
      * **apiGroup** ``Optional[str]`` - APIGroup is the group for the resource being referenced. It is empty for the
        core API. This matches the group in the APIVersion that is used when creating
        the resources.
      * **namespace** ``Optional[str]`` - Namespace that contains the referenced resource. Must be empty for
        cluster-scoped resources and non-empty for namespaced resources.
    """
    kind: 'str'
    name: 'str'
    apiGroup: 'Optional[str]' = None
    namespace: 'Optional[str]' = None





# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import core_v1
from . import meta_v1


@dataclass
class Endpoint(DictMixin):
    """Endpoint represents a single logical "backend" implementing a service.

      **parameters**

      * **addresses** ``List[str]`` - addresses of this endpoint. The contents of this field are interpreted
        according to the corresponding EndpointSlice addressType field. Consumers must
        handle different types of addresses in the context of their own capabilities.
        This must contain at least one address but no more than 100. These are all
        assumed to be fungible and clients may choose to only use the first element.
        Refer to: https://issue.k8s.io/106267
      * **conditions** ``Optional[EndpointConditions]`` - conditions contains information about the current status of the endpoint.
      * **deprecatedTopology** ``Optional[dict]`` - deprecatedTopology contains topology information part of the v1beta1 API. This
        field is deprecated, and will be removed when the v1beta1 API is removed (no
        sooner than kubernetes v1.24).  While this field can hold values, it is not
        writable through the v1 API, and any attempts to write to it will be silently
        ignored. Topology information can be found in the zone and nodeName fields
        instead.
      * **hints** ``Optional[EndpointHints]`` - hints contains information associated with how an endpoint should be consumed.
      * **hostname** ``Optional[str]`` - hostname of this endpoint. This field may be used by consumers of endpoints to
        distinguish endpoints from each other (e.g. in DNS names). Multiple endpoints
        which use the same hostname should be considered fungible (e.g. multiple A
        values in DNS). Must be lowercase and pass DNS Label (RFC 1123) validation.
      * **nodeName** ``Optional[str]`` - nodeName represents the name of the Node hosting this endpoint. This can be
        used to determine endpoints local to a Node.
      * **targetRef** ``Optional[core_v1.ObjectReference]`` - targetRef is a reference to a Kubernetes object that represents this endpoint.
      * **zone** ``Optional[str]`` - zone is the name of the Zone this endpoint exists in.
    """
    addresses: 'List[str]'
    conditions: 'Optional[EndpointConditions]' = None
    deprecatedTopology: 'Optional[dict]' = None
    hints: 'Optional[EndpointHints]' = None
    hostname: 'Optional[str]' = None
    nodeName: 'Optional[str]' = None
    targetRef: 'Optional[core_v1.ObjectReference]' = None
    zone: 'Optional[str]' = None


@dataclass
class EndpointConditions(DictMixin):
    """EndpointConditions represents the current condition of an endpoint.

      **parameters**

      * **ready** ``Optional[bool]`` - ready indicates that this endpoint is prepared to receive traffic, according
        to whatever system is managing the endpoint. A nil value indicates an unknown
        state. In most cases consumers should interpret this unknown state as ready.
        For compatibility reasons, ready should never be "true" for terminating
        endpoints.
      * **serving** ``Optional[bool]`` - serving is identical to ready except that it is set regardless of the
        terminating state of endpoints. This condition should be set to true for a
        ready endpoint that is terminating. If nil, consumers should defer to the
        ready condition.
      * **terminating** ``Optional[bool]`` - terminating indicates that this endpoint is terminating. A nil value indicates
        an unknown state. Consumers should interpret this unknown state to mean that
        the endpoint is not terminating.
    """
    ready: 'Optional[bool]' = None
    serving: 'Optional[bool]' = None
    terminating: 'Optional[bool]' = None


@dataclass
class EndpointHints(DictMixin):
    """EndpointHints provides hints describing how an endpoint should be consumed.

      **parameters**

      * **forZones** ``Optional[List[ForZone]]`` - forZones indicates the zone(s) this endpoint should be consumed by to enable
        topology aware routing.
    """
    forZones: 'Optional[List[ForZone]]' = None


@dataclass
class EndpointPort(DictMixin):
    """EndpointPort represents a Port used by an EndpointSlice

      **parameters**

      * **appProtocol** ``Optional[str]`` - The application protocol for this port. This field follows standard Kubernetes
        label syntax. Un-prefixed names are reserved for IANA standard service names
        (as per RFC-6335 and https://www.iana.org/assignments/service-names).
        Non-standard protocols should use prefixed names such as
        mycompany.com/my-custom-protocol.
      * **name** ``Optional[str]`` - The name of this port. All ports in an EndpointSlice must have a unique name.
        If the EndpointSlice is dervied from a Kubernetes service, this corresponds to
        the Service.ports[].name. Name must either be an empty string or pass
        DNS_LABEL validation: * must be no more than 63 characters long. * must
        consist of lower case alphanumeric characters or '-'. * must start and end
        with an alphanumeric character. Default is empty string.
      * **port** ``Optional[int]`` - The port number of the endpoint. If this is not specified, ports are not
        restricted and must be interpreted in the context of the specific consumer.
      * **protocol** ``Optional[str]`` - The IP protocol for this port. Must be UDP, TCP, or SCTP. Default is TCP.
    """
    appProtocol: 'Optional[str]' = None
    name: 'Optional[str]' = None
    port: 'Optional[int]' = None
    protocol: 'Optional[str]' = None


@dataclass
class EndpointSlice(DictMixin):
    """EndpointSlice represents a subset of the endpoints that implement a service.
      For a given service there may be multiple EndpointSlice objects, selected by
      labels, which must be joined to produce the full set of endpoints.

      **parameters**

      * **addressType** ``str`` - addressType specifies the type of address carried by this EndpointSlice. All
        addresses in this slice must be the same type. This field is immutable after
        creation. The following address types are currently supported: * IPv4:
        Represents an IPv4 Address. * IPv6: Represents an IPv6 Address. * FQDN:
        Represents a Fully Qualified Domain Name.
      * **endpoints** ``List[Endpoint]`` - endpoints is a list of unique endpoints in this slice. Each slice may include
        a maximum of 1000 endpoints.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata.
      * **ports** ``Optional[List[EndpointPort]]`` - ports specifies the list of network ports exposed by each endpoint in this
        slice. Each port must have a unique name. When ports is empty, it indicates
        that there are no defined ports. When a port is defined with a nil port value,
        it indicates "all ports". Each slice may include a maximum of 100 ports.
    """
    addressType: 'str'
    endpoints: 'List[Endpoint]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    ports: 'Optional[List[EndpointPort]]' = None


@dataclass
class EndpointSliceList(DictMixin):
    """EndpointSliceList represents a list of endpoint slices

      **parameters**

      * **items** ``List[EndpointSlice]`` - List of endpoint slices
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata.
    """
    items: 'List[EndpointSlice]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ForZone(DictMixin):
    """ForZone provides information about which zones should consume this endpoint.

      **parameters**

      * **name** ``str`` - name represents the name of the zone.
    """
    name: 'str'





# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import meta_v1
from . import authentication_v1


@dataclass
class SelfSubjectReview(DictMixin):
    """SelfSubjectReview contains the user information that the kube-apiserver has
      about the user making this request. When using impersonation, users will
      receive the user info of the user being impersonated.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **status** ``Optional[SelfSubjectReviewStatus]`` - Status is filled in by the server with the user attributes.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[SelfSubjectReviewStatus]' = None


@dataclass
class SelfSubjectReviewStatus(DictMixin):
    """SelfSubjectReviewStatus is filled by the kube-apiserver and sent back to a
      user.

      **parameters**

      * **userInfo** ``Optional[authentication_v1.UserInfo]`` - User attributes of the user making this request.
    """
    userInfo: 'Optional[authentication_v1.UserInfo]' = None





# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import meta_v1


@dataclass
class MatchResources(DictMixin):
    """MatchResources decides whether to run the admission control policy on an
      object based on whether it meets the match criteria. The exclude rules take
      precedence over include rules (if a resource matches both, it is excluded)

      **parameters**

      * **excludeResourceRules** ``Optional[List[NamedRuleWithOperations]]`` - ExcludeResourceRules describes what operations on what resources/subresources
        the ValidatingAdmissionPolicy should not care about. The exclude rules take
        precedence over include rules (if a resource matches both, it is excluded)
      * **matchPolicy** ``Optional[str]`` - matchPolicy defines how the "MatchResources" list is used to match incoming
        requests. Allowed values are "Exact" or "Equivalent".
        - Exact: match a request only if it exactly matches a specified rule. For
        example, if deployments can be modified via apps/v1, apps/v1beta1, and
        extensions/v1beta1, but "rules" only included `apiGroups:["apps"],
        apiVersions:["v1"], resources: ["deployments"]`, a request to apps/v1beta1 or
        extensions/v1beta1 would not be sent to the ValidatingAdmissionPolicy.
        - Equivalent: match a request if modifies a resource listed in rules, even via
        another API group or version. For example, if deployments can be modified via
        apps/v1, apps/v1beta1, and extensions/v1beta1, and "rules" only included
        `apiGroups:["apps"], apiVersions:["v1"], resources: ["deployments"]`, a
        request to apps/v1beta1 or extensions/v1beta1 would be converted to apps/v1
        and sent to the ValidatingAdmissionPolicy.
        Defaults to "Equivalent"
      * **namespaceSelector** ``Optional[meta_v1.LabelSelector]`` - NamespaceSelector decides whether to run the admission control policy on an
        object based on whether the namespace for that object matches the selector. If
        the object itself is a namespace, the matching is performed on
        object.metadata.labels. If the object is another cluster scoped resource, it
        never skips the policy.
        For example, to run the webhook on any objects whose namespace is not
        associated with "runlevel" of "0" or "1";  you will set the selector as
        follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "runlevel",
              "operator": "NotIn",
              "values": [
                "0",
                "1"
              ]
            }
          ]
        }
        If instead you want to only run the policy on any objects whose namespace is
        associated with the "environment" of "prod" or "staging"; you will set the
        selector as follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "environment",
              "operator": "In",
              "values": [
                "prod",
                "staging"
              ]
            }
          ]
        }
        See https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/
        for more examples of label selectors.
        Default to the empty LabelSelector, which matches everything.
      * **objectSelector** ``Optional[meta_v1.LabelSelector]`` - ObjectSelector decides whether to run the validation based on if the object
        has matching labels. objectSelector is evaluated against both the oldObject
        and newObject that would be sent to the cel validation, and is considered to
        match if either object matches the selector. A null object (oldObject in the
        case of create, or newObject in the case of delete) or an object that cannot
        have labels (like a DeploymentRollback or a PodProxyOptions object) is not
        considered to match. Use the object selector only if the webhook is opt-in,
        because end users may skip the admission webhook by setting the labels.
        Default to the empty LabelSelector, which matches everything.
      * **resourceRules** ``Optional[List[NamedRuleWithOperations]]`` - ResourceRules describes what operations on what resources/subresources the
        ValidatingAdmissionPolicy matches. The policy cares about an operation if it
        matches _any_ Rule.
    """
    excludeResourceRules: 'Optional[List[NamedRuleWithOperations]]' = None
    matchPolicy: 'Optional[str]' = None
    namespaceSelector: 'Optional[meta_v1.LabelSelector]' = None
    objectSelector: 'Optional[meta_v1.LabelSelector]' = None
    resourceRules: 'Optional[List[NamedRuleWithOperations]]' = None


@dataclass
class NamedRuleWithOperations(DictMixin):
    """NamedRuleWithOperations is a tuple of Operations and Resources with
      ResourceNames.

      **parameters**

      * **apiGroups** ``Optional[List[str]]`` - APIGroups is the API groups the resources belong to. '*' is all groups. If '*'
        is present, the length of the slice must be one. Required.
      * **apiVersions** ``Optional[List[str]]`` - APIVersions is the API versions the resources belong to. '*' is all versions.
        If '*' is present, the length of the slice must be one. Required.
      * **operations** ``Optional[List[str]]`` - Operations is the operations the admission hook cares about - CREATE, UPDATE,
        DELETE, CONNECT or * for all of those operations and any future admission
        operations that are added. If '*' is present, the length of the slice must be
        one. Required.
      * **resourceNames** ``Optional[List[str]]`` - ResourceNames is an optional white list of names that the rule applies to.  An
        empty set means that everything is allowed.
      * **resources** ``Optional[List[str]]`` - Resources is a list of resources this rule applies to.
        For example: 'pods' means pods. 'pods/log' means the log subresource of pods.
        '*' means all resources, but not subresources. 'pods/*' means all subresources
        of pods. '*/scale' means all scale subresources. '*/*' means all resources and
        their subresources.
        If wildcard is present, the validation rule will ensure resources do not
        overlap with each other.
        Depending on the enclosing object, subresources might not be allowed.
        Required.
      * **scope** ``Optional[str]`` - scope specifies the scope of this rule. Valid values are "Cluster",
        "Namespaced", and "*" "Cluster" means that only cluster-scoped resources will
        match this rule. Namespace API objects are cluster-scoped. "Namespaced" means
        that only namespaced resources will match this rule. "*" means that there are
        no scope restrictions. Subresources match the scope of their parent resource.
        Default is "*".
    """
    apiGroups: 'Optional[List[str]]' = None
    apiVersions: 'Optional[List[str]]' = None
    operations: 'Optional[List[str]]' = None
    resourceNames: 'Optional[List[str]]' = None
    resources: 'Optional[List[str]]' = None
    scope: 'Optional[str]' = None


@dataclass
class ParamKind(DictMixin):
    """ParamKind is a tuple of Group Kind and Version.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion is the API group version the resources belong to. In format of
        "group/version". Required.
      * **kind** ``Optional[str]`` - Kind is the API kind the resources belong to. Required.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None


@dataclass
class ParamRef(DictMixin):
    """ParamRef references a parameter resource

      **parameters**

      * **name** ``Optional[str]`` - Name of the resource being referenced.
      * **namespace** ``Optional[str]`` - Namespace of the referenced resource. Should be empty for the cluster-scoped
        resources
    """
    name: 'Optional[str]' = None
    namespace: 'Optional[str]' = None


@dataclass
class ValidatingAdmissionPolicy(DictMixin):
    """ValidatingAdmissionPolicy describes the definition of an admission validation
      policy that accepts or rejects an object without changing it.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.
      * **spec** ``Optional[ValidatingAdmissionPolicySpec]`` - Specification of the desired behavior of the ValidatingAdmissionPolicy.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[ValidatingAdmissionPolicySpec]' = None


@dataclass
class ValidatingAdmissionPolicyBinding(DictMixin):
    """ValidatingAdmissionPolicyBinding binds the ValidatingAdmissionPolicy with
      paramerized resources. ValidatingAdmissionPolicyBinding and parameter CRDs
      together define how cluster administrators configure policies for clusters.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.
      * **spec** ``Optional[ValidatingAdmissionPolicyBindingSpec]`` - Specification of the desired behavior of the ValidatingAdmissionPolicyBinding.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[ValidatingAdmissionPolicyBindingSpec]' = None


@dataclass
class ValidatingAdmissionPolicyBindingList(DictMixin):
    """ValidatingAdmissionPolicyBindingList is a list of
      ValidatingAdmissionPolicyBinding.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **items** ``Optional[List[ValidatingAdmissionPolicyBinding]]`` - List of PolicyBinding.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    apiVersion: 'Optional[str]' = None
    items: 'Optional[List[ValidatingAdmissionPolicyBinding]]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ValidatingAdmissionPolicyBindingSpec(DictMixin):
    """ValidatingAdmissionPolicyBindingSpec is the specification of the
      ValidatingAdmissionPolicyBinding.

      **parameters**

      * **matchResources** ``Optional[MatchResources]`` - MatchResources declares what resources match this binding and will be
        validated by it. Note that this is intersected with the policy's
        matchConstraints, so only requests that are matched by the policy can be
        selected by this. If this is unset, all resources matched by the policy are
        validated by this binding When resourceRules is unset, it does not constrain
        resource matching. If a resource is matched by the other fields of this
        object, it will be validated. Note that this is differs from
        ValidatingAdmissionPolicy matchConstraints, where resourceRules are required.
      * **paramRef** ``Optional[ParamRef]`` - ParamRef specifies the parameter resource used to configure the admission
        control policy. It should point to a resource of the type specified in
        ParamKind of the bound ValidatingAdmissionPolicy. If the policy specifies a
        ParamKind and the resource referred to by ParamRef does not exist, this
        binding is considered mis-configured and the FailurePolicy of the
        ValidatingAdmissionPolicy applied.
      * **policyName** ``Optional[str]`` - PolicyName references a ValidatingAdmissionPolicy name which the
        ValidatingAdmissionPolicyBinding binds to. If the referenced resource does not
        exist, this binding is considered invalid and will be ignored Required.
    """
    matchResources: 'Optional[MatchResources]' = None
    paramRef: 'Optional[ParamRef]' = None
    policyName: 'Optional[str]' = None


@dataclass
class ValidatingAdmissionPolicyList(DictMixin):
    """ValidatingAdmissionPolicyList is a list of ValidatingAdmissionPolicy.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **items** ``Optional[List[ValidatingAdmissionPolicy]]`` - List of ValidatingAdmissionPolicy.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    apiVersion: 'Optional[str]' = None
    items: 'Optional[List[ValidatingAdmissionPolicy]]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ValidatingAdmissionPolicySpec(DictMixin):
    """ValidatingAdmissionPolicySpec is the specification of the desired behavior of
      the AdmissionPolicy.

      **parameters**

      * **validations** ``List[Validation]`` - Validations contain CEL expressions which is used to apply the validation. A
        minimum of one validation is required for a policy definition. Required.
      * **failurePolicy** ``Optional[str]`` - FailurePolicy defines how to handle failures for the admission policy.
        Failures can occur from invalid or mis-configured policy definitions or
        bindings. A policy is invalid if spec.paramKind refers to a non-existent Kind.
        A binding is invalid if spec.paramRef.name refers to a non-existent resource.
        Allowed values are Ignore or Fail. Defaults to Fail.
      * **matchConstraints** ``Optional[MatchResources]`` - MatchConstraints specifies what resources this policy is designed to validate.
        The AdmissionPolicy cares about a request if it matches _all_ Constraints.
        However, in order to prevent clusters from being put into an unstable state
        that cannot be recovered from via the API ValidatingAdmissionPolicy cannot
        match ValidatingAdmissionPolicy and ValidatingAdmissionPolicyBinding.
        Required.
      * **paramKind** ``Optional[ParamKind]`` - ParamKind specifies the kind of resources used to parameterize this policy. If
        absent, there are no parameters for this policy and the param CEL variable
        will not be provided to validation expressions. If ParamKind refers to a
        non-existent kind, this policy definition is mis-configured and the
        FailurePolicy is applied. If paramKind is specified but paramRef is unset in
        ValidatingAdmissionPolicyBinding, the params variable will be null.
    """
    validations: 'List[Validation]'
    failurePolicy: 'Optional[str]' = None
    matchConstraints: 'Optional[MatchResources]' = None
    paramKind: 'Optional[ParamKind]' = None


@dataclass
class Validation(DictMixin):
    """Validation specifies the CEL expression which is used to apply the validation.

      **parameters**

      * **expression** ``str`` - Expression represents the expression which will be evaluated by CEL. ref:
        https://github.com/google/cel-spec CEL expressions have access to the contents
        of the Admission request/response, organized into CEL variables as well as
        some other useful variables:
        'object' - The object from the incoming request. The value is null for DELETE
        requests. 'oldObject' - The existing object. The value is null for CREATE
        requests. 'request' - Attributes of the admission
        request([ref](/pkg/apis/admission/types.go#AdmissionRequest)). 'params' -
        Parameter resource referred to by the policy binding being evaluated. Only
        populated if the policy has a ParamKind.
        The `apiVersion`, `kind`, `metadata.name` and `metadata.generateName` are
        always accessible from the root of the object. No other metadata properties
        are accessible.
        Only property names of the form `[a-zA-Z_.-/][a-zA-Z0-9_.-/]*` are accessible.
        Accessible property names are escaped according to the following rules when
        accessed in the expression: - '__' escapes to '__underscores__' - '.' escapes
        to '__dot__' - '-' escapes to '__dash__' - '/' escapes to '__slash__' -
        Property names that exactly match a CEL RESERVED keyword escape to
        '__{keyword}__'. The keywords are:
        	  "true", "false", "null", "in", "as", "break", "const", "continue", "else",
        "for", "function", "if",
        	  "import", "let", "loop", "package", "namespace", "return".
        Examples:
          - Expression accessing a property named "namespace": {"Expression":
        "object.__namespace__ > 0"}
          - Expression accessing a property named "x-prop": {"Expression":
        "object.x__dash__prop > 0"}
          - Expression accessing a property named "redact__d": {"Expression":
        "object.redact__underscores__d > 0"}
        Equality on arrays with list type of 'set' or 'map' ignores element order,
        i.e. [1, 2] == [2, 1]. Concatenation on arrays with x-kubernetes-list-type use
        the semantics of the list type:
          - 'set': `X + Y` performs a union where the array positions of all elements
        in `X` are preserved and
            non-intersecting elements in `Y` are appended, retaining their partial
        order.
          - 'map': `X + Y` performs a merge where the array positions of all keys in
        `X` are preserved but the values
            are overwritten by values in `Y` when the key sets of `X` and `Y`
        intersect. Elements in `Y` with
            non-intersecting keys are appended, retaining their partial order.
        Required.
      * **message** ``Optional[str]`` - Message represents the message displayed when validation fails. The message is
        required if the Expression contains line breaks. The message must not contain
        line breaks. If unset, the message is "failed rule: {Rule}". e.g. "must be a
        URL with the host matching spec.host" If the Expression contains line breaks.
        Message is required. The message must not contain line breaks. If unset, the
        message is "failed Expression: {Expression}".
      * **reason** ``Optional[str]`` - Reason represents a machine-readable description of why this validation
        failed. If this is the first validation in the list to fail, this reason, as
        well as the corresponding HTTP response code, are used in the HTTP response to
        the client. The currently supported reasons are: "Unauthorized", "Forbidden",
        "Invalid", "RequestEntityTooLarge". If not set, StatusReasonInvalid is used in
        the response to the client.
    """
    expression: 'str'
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None





# autogenerated module
from typing import ClassVar

from ..core import resource as res
from ..models import authentication_v1alpha1 as m_authentication_v1alpha1


class SelfSubjectReview(res.GlobalResource, m_authentication_v1alpha1.SelfSubjectReview):
    """* **Extends**: ``models.authentication_v1alpha1.SelfSubjectReview``
       * **Type**: Global Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('authentication.k8s.io', 'v1alpha1', 'SelfSubjectReview'),
        plural='selfsubjectreviews',
        verbs=['post'],
    )




# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import meta_v1
from . import util_intstr


@dataclass
class Eviction(DictMixin):
    """Eviction evicts a pod from its node subject to certain policies and safety
      constraints. This is a subresource of Pod.  A request to cause such an
      eviction is created by POSTing to .../pods/<pod name>/evictions.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **deleteOptions** ``Optional[meta_v1.DeleteOptions]`` - DeleteOptions may be provided
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - ObjectMeta describes the pod that is being evicted.
    """
    apiVersion: 'Optional[str]' = None
    deleteOptions: 'Optional[meta_v1.DeleteOptions]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class PodDisruptionBudget(DictMixin):
    """PodDisruptionBudget is an object to define the max disruption that can be
      caused to a collection of pods

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[PodDisruptionBudgetSpec]`` - Specification of the desired behavior of the PodDisruptionBudget.
      * **status** ``Optional[PodDisruptionBudgetStatus]`` - Most recently observed status of the PodDisruptionBudget.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[PodDisruptionBudgetSpec]' = None
    status: 'Optional[PodDisruptionBudgetStatus]' = None


@dataclass
class PodDisruptionBudgetList(DictMixin):
    """PodDisruptionBudgetList is a collection of PodDisruptionBudgets.

      **parameters**

      * **items** ``List[PodDisruptionBudget]`` - Items is a list of PodDisruptionBudgets
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[PodDisruptionBudget]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class PodDisruptionBudgetSpec(DictMixin):
    """PodDisruptionBudgetSpec is a description of a PodDisruptionBudget.

      **parameters**

      * **maxUnavailable** ``Optional[util_intstr.IntOrString]`` - An eviction is allowed if at most "maxUnavailable" pods selected by "selector"
        are unavailable after the eviction, i.e. even in absence of the evicted pod.
        For example, one can prevent all voluntary evictions by specifying 0. This is
        a mutually exclusive setting with "minAvailable".
      * **minAvailable** ``Optional[util_intstr.IntOrString]`` - An eviction is allowed if at least "minAvailable" pods selected by "selector"
        will still be available after the eviction, i.e. even in the absence of the
        evicted pod.  So for example you can prevent all voluntary evictions by
        specifying "100%".
      * **selector** ``Optional[meta_v1.LabelSelector]`` - Label query over pods whose evictions are managed by the disruption budget. A
        null selector will match no pods, while an empty ({}) selector will select all
        pods within the namespace.
      * **unhealthyPodEvictionPolicy** ``Optional[str]`` - UnhealthyPodEvictionPolicy defines the criteria for when unhealthy pods should
        be considered for eviction. Current implementation considers healthy pods, as
        pods that have status.conditions item with type="Ready",status="True".
        Valid policies are IfHealthyBudget and AlwaysAllow. If no policy is specified,
        the default behavior will be used, which corresponds to the IfHealthyBudget
        policy.
        IfHealthyBudget policy means that running pods (status.phase="Running"), but
        not yet healthy can be evicted only if the guarded application is not
        disrupted (status.currentHealthy is at least equal to status.desiredHealthy).
        Healthy pods will be subject to the PDB for eviction.
        AlwaysAllow policy means that all running pods (status.phase="Running"), but
        not yet healthy are considered disrupted and can be evicted regardless of
        whether the criteria in a PDB is met. This means perspective running pods of a
        disrupted application might not get a chance to become healthy. Healthy pods
        will be subject to the PDB for eviction.
        Additional policies may be added in the future. Clients making eviction
        decisions should disallow eviction of unhealthy pods if they encounter an
        unrecognized policy in this field.
        This field is alpha-level. The eviction API uses this field when the feature
        gate PDBUnhealthyPodEvictionPolicy is enabled (disabled by default).
    """
    maxUnavailable: 'Optional[util_intstr.IntOrString]' = None
    minAvailable: 'Optional[util_intstr.IntOrString]' = None
    selector: 'Optional[meta_v1.LabelSelector]' = None
    unhealthyPodEvictionPolicy: 'Optional[str]' = None


@dataclass
class PodDisruptionBudgetStatus(DictMixin):
    """PodDisruptionBudgetStatus represents information about the status of a
      PodDisruptionBudget. Status may trail the actual state of a system.

      **parameters**

      * **currentHealthy** ``int`` - current number of healthy pods
      * **desiredHealthy** ``int`` - minimum desired number of healthy pods
      * **disruptionsAllowed** ``int`` - Number of pod disruptions that are currently allowed.
      * **expectedPods** ``int`` - total number of pods counted by this disruption budget
      * **conditions** ``Optional[List[meta_v1.Condition]]`` - Conditions contain conditions for PDB. The disruption controller sets the
        DisruptionAllowed condition. The following are known values for the reason
        field (additional reasons could be added in the future): - SyncFailed: The
        controller encountered an error and wasn't able to compute
                      the number of allowed disruptions. Therefore no disruptions are
                      allowed and the status of the condition will be False.
        - InsufficientPods: The number of pods are either at or below the number
                            required by the PodDisruptionBudget. No disruptions are
                            allowed and the status of the condition will be False.
        - SufficientPods: There are more pods than required by the
        PodDisruptionBudget.
                          The condition will be True, and the number of allowed
                          disruptions are provided by the disruptionsAllowed property.
      * **disruptedPods** ``Optional[dict]`` - DisruptedPods contains information about pods whose eviction was processed by
        the API server eviction subresource handler but has not yet been observed by
        the PodDisruptionBudget controller. A pod will be in this map from the time
        when the API server processed the eviction request to the time when the pod is
        seen by PDB controller as having been marked for deletion (or after a
        timeout). The key in the map is the name of the pod and the value is the time
        when the API server processed the eviction request. If the deletion didn't
        occur and a pod is still there it will be removed from the list automatically
        by PodDisruptionBudget controller after some time. If everything goes smooth
        this map should be empty for the most of the time. Large number of entries in
        the map may indicate problems with pod deletions.
      * **observedGeneration** ``Optional[int]`` - Most recent generation observed when updating this PDB status.
        DisruptionsAllowed and other status information is valid only if
        observedGeneration equals to PDB's object generation.
    """
    currentHealthy: 'int'
    desiredHealthy: 'int'
    disruptionsAllowed: 'int'
    expectedPods: 'int'
    conditions: 'Optional[List[meta_v1.Condition]]' = None
    disruptedPods: 'Optional[dict]' = None
    observedGeneration: 'Optional[int]' = None





# autogenerated module
from typing import ClassVar

from ..core import resource as res
from ..models import authentication_v1 as m_authentication_v1
from ..models import autoscaling_v1 as m_autoscaling_v1
from ..models import core_v1 as m_core_v1
from ..models import policy_v1 as m_policy_v1


class ComponentStatus(res.GlobalResource, m_core_v1.ComponentStatus):
    """* **Extends**: ``models.core_v1.ComponentStatus``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `list`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'ComponentStatus'),
        plural='componentstatuses',
        verbs=['get', 'list', 'watch'],
    )


class ConfigMap(res.NamespacedResourceG, m_core_v1.ConfigMap):
    """* **Extends**: ``models.core_v1.ConfigMap``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'ConfigMap'),
        plural='configmaps',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )


class Endpoints(res.NamespacedResourceG, m_core_v1.Endpoints):
    """* **Extends**: ``models.core_v1.Endpoints``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Endpoints'),
        plural='endpoints',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )


class Event(res.NamespacedResourceG, m_core_v1.Event):
    """* **Extends**: ``models.core_v1.Event``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Event'),
        plural='events',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )


class LimitRange(res.NamespacedResourceG, m_core_v1.LimitRange):
    """* **Extends**: ``models.core_v1.LimitRange``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'LimitRange'),
        plural='limitranges',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )


class NamespaceFinalize(res.GlobalSubResource, m_core_v1.Namespace):
    """* **Extends**: ``models.core_v1.Namespace``
       * **Type**: Global Resource
       * **Accepted client methods**: `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Namespace'),
        parent=res.ResourceDef('', 'v1', 'Namespace'),
        plural='namespaces',
        verbs=['put'],
        action='finalize',
    )


class NamespaceStatus(res.GlobalSubResource, m_core_v1.Namespace):
    """* **Extends**: ``models.core_v1.Namespace``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Namespace'),
        parent=res.ResourceDef('', 'v1', 'Namespace'),
        plural='namespaces',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class Namespace(res.GlobalResource, m_core_v1.Namespace):
    """* **Extends**: ``models.core_v1.Namespace``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `get`, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Finalize**: ``NamespaceFinalize``
       * **Status**: ``NamespaceStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Namespace'),
        plural='namespaces',
        verbs=['delete', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Finalize: ClassVar = NamespaceFinalize
    Status: ClassVar = NamespaceStatus


class Binding(res.NamespacedResource, m_core_v1.Binding):
    """* **Extends**: ``models.core_v1.Binding``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Binding'),
        plural='bindings',
        verbs=['post'],
    )


class PersistentVolumeClaimStatus(res.NamespacedSubResource, m_core_v1.PersistentVolumeClaim):
    """* **Extends**: ``models.core_v1.PersistentVolumeClaim``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'PersistentVolumeClaim'),
        parent=res.ResourceDef('', 'v1', 'PersistentVolumeClaim'),
        plural='persistentvolumeclaims',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class PersistentVolumeClaim(res.NamespacedResourceG, m_core_v1.PersistentVolumeClaim):
    """* **Extends**: ``models.core_v1.PersistentVolumeClaim``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``PersistentVolumeClaimStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'PersistentVolumeClaim'),
        plural='persistentvolumeclaims',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = PersistentVolumeClaimStatus


class PodBinding(res.NamespacedSubResource, m_core_v1.Binding):
    """* **Extends**: ``models.core_v1.Binding``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Binding'),
        parent=res.ResourceDef('', 'v1', 'Pod'),
        plural='pods',
        verbs=['post'],
        action='binding',
    )


class PodEphemeralcontainers(res.NamespacedSubResource, m_core_v1.Pod):
    """* **Extends**: ``models.core_v1.Pod``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Pod'),
        parent=res.ResourceDef('', 'v1', 'Pod'),
        plural='pods',
        verbs=['get', 'patch', 'put'],
        action='ephemeralcontainers',
    )


class PodEviction(res.NamespacedSubResource, m_policy_v1.Eviction):
    """* **Extends**: ``models.policy_v1.Eviction``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('policy', 'v1', 'Eviction'),
        parent=res.ResourceDef('', 'v1', 'Pod'),
        plural='pods',
        verbs=['post'],
        action='eviction',
    )


class PodLog(res.NamespacedSubResource):
    """* **Extends**: ``models.mespacedSubResource``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Pod'),
        parent=res.ResourceDef('', 'v1', 'Pod'),
        plural='pods',
        verbs=['get'],
        action='log',
    )


class PodStatus(res.NamespacedSubResource, m_core_v1.Pod):
    """* **Extends**: ``models.core_v1.Pod``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Pod'),
        parent=res.ResourceDef('', 'v1', 'Pod'),
        plural='pods',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class Pod(res.NamespacedResourceG, m_core_v1.Pod):
    """* **Extends**: ``models.core_v1.Pod``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Binding**: ``PodBinding``
       * **Ephemeralcontainers**: ``PodEphemeralcontainers``
       * **Eviction**: ``PodEviction``
       * **Log**: ``PodLog``
       * **Status**: ``PodStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Pod'),
        plural='pods',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Binding: ClassVar = PodBinding
    Ephemeralcontainers: ClassVar = PodEphemeralcontainers
    Eviction: ClassVar = PodEviction
    Log: ClassVar = PodLog
    Status: ClassVar = PodStatus


class PodTemplate(res.NamespacedResourceG, m_core_v1.PodTemplate):
    """* **Extends**: ``models.core_v1.PodTemplate``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'PodTemplate'),
        plural='podtemplates',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )


class ReplicationControllerScale(res.NamespacedSubResource, m_autoscaling_v1.Scale):
    """* **Extends**: ``models.autoscaling_v1.Scale``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('autoscaling', 'v1', 'Scale'),
        parent=res.ResourceDef('', 'v1', 'ReplicationController'),
        plural='replicationcontrollers',
        verbs=['get', 'patch', 'put'],
        action='scale',
    )


class ReplicationControllerStatus(res.NamespacedSubResource, m_core_v1.ReplicationController):
    """* **Extends**: ``models.core_v1.ReplicationController``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'ReplicationController'),
        parent=res.ResourceDef('', 'v1', 'ReplicationController'),
        plural='replicationcontrollers',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class ReplicationController(res.NamespacedResourceG, m_core_v1.ReplicationController):
    """* **Extends**: ``models.core_v1.ReplicationController``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Scale**: ``ReplicationControllerScale``
       * **Status**: ``ReplicationControllerStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'ReplicationController'),
        plural='replicationcontrollers',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Scale: ClassVar = ReplicationControllerScale
    Status: ClassVar = ReplicationControllerStatus


class ResourceQuotaStatus(res.NamespacedSubResource, m_core_v1.ResourceQuota):
    """* **Extends**: ``models.core_v1.ResourceQuota``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'ResourceQuota'),
        parent=res.ResourceDef('', 'v1', 'ResourceQuota'),
        plural='resourcequotas',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class ResourceQuota(res.NamespacedResourceG, m_core_v1.ResourceQuota):
    """* **Extends**: ``models.core_v1.ResourceQuota``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``ResourceQuotaStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'ResourceQuota'),
        plural='resourcequotas',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = ResourceQuotaStatus


class Secret(res.NamespacedResourceG, m_core_v1.Secret):
    """* **Extends**: ``models.core_v1.Secret``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Secret'),
        plural='secrets',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )


class ServiceAccountToken(res.NamespacedSubResource, m_authentication_v1.TokenRequest):
    """* **Extends**: ``models.authentication_v1.TokenRequest``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('authentication.k8s.io', 'v1', 'TokenRequest'),
        parent=res.ResourceDef('', 'v1', 'ServiceAccount'),
        plural='serviceaccounts',
        verbs=['post'],
        action='token',
    )


class ServiceAccount(res.NamespacedResourceG, m_core_v1.ServiceAccount):
    """* **Extends**: ``models.core_v1.ServiceAccount``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Token**: ``ServiceAccountToken``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'ServiceAccount'),
        plural='serviceaccounts',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Token: ClassVar = ServiceAccountToken


class ServiceStatus(res.NamespacedSubResource, m_core_v1.Service):
    """* **Extends**: ``models.core_v1.Service``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Service'),
        parent=res.ResourceDef('', 'v1', 'Service'),
        plural='services',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class Service(res.NamespacedResourceG, m_core_v1.Service):
    """* **Extends**: ``models.core_v1.Service``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list` all, `watch` all, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``ServiceStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Service'),
        plural='services',
        verbs=['delete', 'deletecollection', 'get', 'global_list', 'global_watch', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = ServiceStatus


class NodeStatus(res.GlobalSubResource, m_core_v1.Node):
    """* **Extends**: ``models.core_v1.Node``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Node'),
        parent=res.ResourceDef('', 'v1', 'Node'),
        plural='nodes',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class Node(res.GlobalResource, m_core_v1.Node):
    """* **Extends**: ``models.core_v1.Node``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``NodeStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'Node'),
        plural='nodes',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = NodeStatus


class PersistentVolumeStatus(res.GlobalSubResource, m_core_v1.PersistentVolume):
    """* **Extends**: ``models.core_v1.PersistentVolume``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'PersistentVolume'),
        parent=res.ResourceDef('', 'v1', 'PersistentVolume'),
        plural='persistentvolumes',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class PersistentVolume(res.GlobalResource, m_core_v1.PersistentVolume):
    """* **Extends**: ``models.core_v1.PersistentVolume``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``PersistentVolumeStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('', 'v1', 'PersistentVolume'),
        plural='persistentvolumes',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = PersistentVolumeStatus




# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import meta_v1
from . import resource


@dataclass
class CSIStorageCapacity(DictMixin):
    """CSIStorageCapacity stores the result of one CSI GetCapacity call. For a given
      StorageClass, this describes the available capacity in a particular topology
      segment.  This can be used when considering where to instantiate new
      PersistentVolumes.
      
      For example this can express things like: - StorageClass "standard" has "1234
      GiB" available in "topology.kubernetes.io/zone=us-east1" - StorageClass
      "localssd" has "10 GiB" available in "kubernetes.io/hostname=knode-abc123"
      
      The following three cases all imply that no capacity is available for a
      certain combination: - no object exists with suitable topology and storage
      class name - such an object exists, but the capacity is unset - such an object
      exists, but the capacity is zero
      
      The producer of these objects can decide which approach is more suitable.
      
      They are consumed by the kube-scheduler when a CSI driver opts into
      capacity-aware scheduling with CSIDriverSpec.StorageCapacity. The scheduler
      compares the MaximumVolumeSize against the requested size of pending volumes
      to filter out unsuitable nodes. If MaximumVolumeSize is unset, it falls back
      to a comparison against the less precise Capacity. If that is also unset, the
      scheduler assumes that capacity is insufficient and tries some other node.

      **parameters**

      * **storageClassName** ``str`` - The name of the StorageClass that the reported capacity applies to. It must
        meet the same requirements as the name of a StorageClass object (non-empty,
        DNS subdomain). If that object no longer exists, the CSIStorageCapacity object
        is obsolete and should be removed by its creator. This field is immutable.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **capacity** ``Optional[resource.Quantity]`` - Capacity is the value reported by the CSI driver in its GetCapacityResponse
        for a GetCapacityRequest with topology and parameters that match the previous
        fields.
        The semantic is currently (CSI spec 1.2) defined as: The available capacity,
        in bytes, of the storage that can be used to provision volumes. If not set,
        that information is currently unavailable.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **maximumVolumeSize** ``Optional[resource.Quantity]`` - MaximumVolumeSize is the value reported by the CSI driver in its
        GetCapacityResponse for a GetCapacityRequest with topology and parameters that
        match the previous fields.
        This is defined since CSI spec 1.4.0 as the largest size that may be used in a
        CreateVolumeRequest.capacity_range.required_bytes field to create a volume
        with the same parameters as those in GetCapacityRequest. The corresponding
        value in the Kubernetes API is ResourceRequirements.Requests in a volume
        claim.
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. The name has no particular meaning. It must be be
        a DNS subdomain (dots allowed, 253 characters). To ensure that there are no
        conflicts with other CSI drivers on the cluster, the recommendation is to use
        csisc-<uuid>, a generated name, or a reverse-domain name which ends with the
        unique CSI driver name.
        Objects are namespaced.
        More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **nodeTopology** ``Optional[meta_v1.LabelSelector]`` - NodeTopology defines which nodes have access to the storage for which capacity
        was reported. If not set, the storage is not accessible from any node in the
        cluster. If empty, the storage is accessible from all nodes. This field is
        immutable.
    """
    storageClassName: 'str'
    apiVersion: 'Optional[str]' = None
    capacity: 'Optional[resource.Quantity]' = None
    kind: 'Optional[str]' = None
    maximumVolumeSize: 'Optional[resource.Quantity]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    nodeTopology: 'Optional[meta_v1.LabelSelector]' = None


@dataclass
class CSIStorageCapacityList(DictMixin):
    """CSIStorageCapacityList is a collection of CSIStorageCapacity objects.

      **parameters**

      * **items** ``List[CSIStorageCapacity]`` - Items is the list of CSIStorageCapacity objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[CSIStorageCapacity]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None





# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import runtime
from typing import Dict
from datetime import datetime


@dataclass
class APIGroup(DictMixin):
    """APIGroup contains the name, the supported versions, and the preferred version
      of a group.

      **parameters**

      * **name** ``str`` - name is the name of the group.
      * **versions** ``List[GroupVersionForDiscovery]`` - versions are the versions supported in this group.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **preferredVersion** ``Optional[GroupVersionForDiscovery]`` - preferredVersion is the version preferred by the API server, which probably is
        the storage version.
      * **serverAddressByClientCIDRs** ``Optional[List[ServerAddressByClientCIDR]]`` - a map of client CIDR to server address that is serving this group. This is to
        help clients reach servers in the most network-efficient way possible. Clients
        can use the appropriate server address as per the CIDR that they match. In
        case of multiple matches, clients should use the longest matching CIDR. The
        server returns only those CIDRs that it thinks that the client can match. For
        example: the master will return an internal IP CIDR only, if the client
        reaches the server using an internal IP. Server looks at X-Forwarded-For
        header or X-Real-Ip header or request.RemoteAddr (in that order) to get the
        client IP.
    """
    name: 'str'
    versions: 'List[GroupVersionForDiscovery]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    preferredVersion: 'Optional[GroupVersionForDiscovery]' = None
    serverAddressByClientCIDRs: 'Optional[List[ServerAddressByClientCIDR]]' = None


@dataclass
class APIGroupList(DictMixin):
    """APIGroupList is a list of APIGroup, to allow clients to discover the API at
      /apis.

      **parameters**

      * **groups** ``List[APIGroup]`` - groups is a list of APIGroup.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    groups: 'List[APIGroup]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None


@dataclass
class APIResource(DictMixin):
    """APIResource specifies the name of a resource and whether it is namespaced.

      **parameters**

      * **kind** ``str`` - kind is the kind for the resource (e.g. 'Foo' is the kind for a resource
        'foo')
      * **name** ``str`` - name is the plural name of the resource.
      * **namespaced** ``bool`` - namespaced indicates if a resource is namespaced or not.
      * **singularName** ``str`` - singularName is the singular name of the resource.  This allows clients to
        handle plural and singular opaquely. The singularName is more correct for
        reporting status on a single item and both singular and plural are allowed
        from the kubectl CLI interface.
      * **verbs** ``List[str]`` - verbs is a list of supported kube verbs (this includes get, list, watch,
        create, update, patch, delete, deletecollection, and proxy)
      * **categories** ``Optional[List[str]]`` - categories is a list of the grouped resources this resource belongs to (e.g.
        'all')
      * **group** ``Optional[str]`` - group is the preferred group of the resource.  Empty implies the group of the
        containing resource list. For subresources, this may have a different value,
        for example: Scale".
      * **shortNames** ``Optional[List[str]]`` - shortNames is a list of suggested short names of the resource.
      * **storageVersionHash** ``Optional[str]`` - The hash value of the storage version, the version this resource is converted
        to when written to the data store. Value must be treated as opaque by clients.
        Only equality comparison on the value is valid. This is an alpha feature and
        may change or be removed in the future. The field is populated by the
        apiserver only if the StorageVersionHash feature gate is enabled. This field
        will remain optional even if it graduates.
      * **version** ``Optional[str]`` - version is the preferred version of the resource.  Empty implies the version
        of the containing resource list For subresources, this may have a different
        value, for example: v1 (while inside a v1beta1 version of the core resource's
        group)".
    """
    kind: 'str'
    name: 'str'
    namespaced: 'bool'
    singularName: 'str'
    verbs: 'List[str]'
    categories: 'Optional[List[str]]' = None
    group: 'Optional[str]' = None
    shortNames: 'Optional[List[str]]' = None
    storageVersionHash: 'Optional[str]' = None
    version: 'Optional[str]' = None


@dataclass
class APIResourceList(DictMixin):
    """APIResourceList is a list of APIResource, it is used to expose the name of the
      resources supported in a specific group and version, and if the resource is
      namespaced.

      **parameters**

      * **groupVersion** ``str`` - groupVersion is the group and version this APIResourceList is for.
      * **resources** ``List[APIResource]`` - resources contains the name of the resources and if they are namespaced.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    groupVersion: 'str'
    resources: 'List[APIResource]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None


@dataclass
class APIVersions(DictMixin):
    """APIVersions lists the versions that are available, to allow clients to
      discover the API at /api, which is the root path of the legacy v1 API.

      **parameters**

      * **serverAddressByClientCIDRs** ``List[ServerAddressByClientCIDR]`` - a map of client CIDR to server address that is serving this group. This is to
        help clients reach servers in the most network-efficient way possible. Clients
        can use the appropriate server address as per the CIDR that they match. In
        case of multiple matches, clients should use the longest matching CIDR. The
        server returns only those CIDRs that it thinks that the client can match. For
        example: the master will return an internal IP CIDR only, if the client
        reaches the server using an internal IP. Server looks at X-Forwarded-For
        header or X-Real-Ip header or request.RemoteAddr (in that order) to get the
        client IP.
      * **versions** ``List[str]`` - versions are the api versions that are available.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    serverAddressByClientCIDRs: 'List[ServerAddressByClientCIDR]'
    versions: 'List[str]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None


@dataclass
class Condition(DictMixin):
    """Condition contains details for one aspect of the current state of this API
      Resource.

      **parameters**

      * **lastTransitionTime** ``Time`` - lastTransitionTime is the last time the condition transitioned from one status
        to another. This should be when the underlying condition changed.  If that is
        not known, then using the time when the API field changed is acceptable.
      * **message** ``str`` - message is a human readable message indicating details about the transition.
        This may be an empty string.
      * **reason** ``str`` - reason contains a programmatic identifier indicating the reason for the
        condition's last transition. Producers of specific condition types may define
        expected values and meanings for this field, and whether the values are
        considered a guaranteed API. The value should be a CamelCase string. This
        field may not be empty.
      * **status** ``str`` - status of the condition, one of True, False, Unknown.
      * **type** ``str`` - type of condition in CamelCase or in foo.example.com/CamelCase.
      * **observedGeneration** ``Optional[int]`` - observedGeneration represents the .metadata.generation that the condition was
        set based upon. For instance, if .metadata.generation is currently 12, but the
        .status.conditions[x].observedGeneration is 9, the condition is out of date
        with respect to the current state of the instance.
    """
    lastTransitionTime: 'Time'
    message: 'str'
    reason: 'str'
    status: 'str'
    type: 'str'
    observedGeneration: 'Optional[int]' = None


@dataclass
class DeleteOptions(DictMixin):
    """DeleteOptions may be provided when deleting an API object.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **dryRun** ``Optional[List[str]]`` - When present, indicates that modifications should not be persisted. An invalid
        or unrecognized dryRun directive will result in an error response and no
        further processing of the request. Valid values are: - All: all dry run stages
        will be processed
      * **gracePeriodSeconds** ``Optional[int]`` - The duration in seconds before the object should be deleted. Value must be
        non-negative integer. The value zero indicates delete immediately. If this
        value is nil, the default grace period for the specified type will be used.
        Defaults to a per object value if not specified. zero means delete
        immediately.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **orphanDependents** ``Optional[bool]`` - Deprecated: please use the PropagationPolicy, this field will be deprecated in
        1.7. Should the dependent objects be orphaned. If true/false, the "orphan"
        finalizer will be added to/removed from the object's finalizers list. Either
        this field or PropagationPolicy may be set, but not both.
      * **preconditions** ``Optional[Preconditions]`` - Must be fulfilled before a deletion is carried out. If not possible, a 409
        Conflict status will be returned.
      * **propagationPolicy** ``Optional[str]`` - Whether and how garbage collection will be performed. Either this field or
        OrphanDependents may be set, but not both. The default policy is decided by
        the existing finalizer set in the metadata.finalizers and the
        resource-specific default policy. Acceptable values are: 'Orphan' - orphan the
        dependents; 'Background' - allow the garbage collector to delete the
        dependents in the background; 'Foreground' - a cascading policy that deletes
        all dependents in the foreground.
    """
    apiVersion: 'Optional[str]' = None
    dryRun: 'Optional[List[str]]' = None
    gracePeriodSeconds: 'Optional[int]' = None
    kind: 'Optional[str]' = None
    orphanDependents: 'Optional[bool]' = None
    preconditions: 'Optional[Preconditions]' = None
    propagationPolicy: 'Optional[str]' = None


FieldsV1 = Dict


@dataclass
class GroupVersionForDiscovery(DictMixin):
    """GroupVersion contains the "group/version" and "version" string of a version.
      It is made a struct to keep extensibility.

      **parameters**

      * **groupVersion** ``str`` - groupVersion specifies the API group and version in the form "group/version"
      * **version** ``str`` - version specifies the version in the form of "version". This is to save the
        clients the trouble of splitting the GroupVersion.
    """
    groupVersion: 'str'
    version: 'str'


@dataclass
class LabelSelector(DictMixin):
    """A label selector is a label query over a set of resources. The result of
      matchLabels and matchExpressions are ANDed. An empty label selector matches
      all objects. A null label selector matches no objects.

      **parameters**

      * **matchExpressions** ``Optional[List[LabelSelectorRequirement]]`` - matchExpressions is a list of label selector requirements. The requirements
        are ANDed.
      * **matchLabels** ``Optional[dict]`` - matchLabels is a map of {key,value} pairs. A single {key,value} in the
        matchLabels map is equivalent to an element of matchExpressions, whose key
        field is "key", the operator is "In", and the values array contains only
        "value". The requirements are ANDed.
    """
    matchExpressions: 'Optional[List[LabelSelectorRequirement]]' = None
    matchLabels: 'Optional[dict]' = None


@dataclass
class LabelSelectorRequirement(DictMixin):
    """A label selector requirement is a selector that contains values, a key, and an
      operator that relates the key and values.

      **parameters**

      * **key** ``str`` - key is the label key that the selector applies to.
      * **operator** ``str`` - operator represents a key's relationship to a set of values. Valid operators
        are In, NotIn, Exists and DoesNotExist.
      * **values** ``Optional[List[str]]`` - values is an array of string values. If the operator is In or NotIn, the
        values array must be non-empty. If the operator is Exists or DoesNotExist, the
        values array must be empty. This array is replaced during a strategic merge
        patch.
    """
    key: 'str'
    operator: 'str'
    values: 'Optional[List[str]]' = None


@dataclass
class ListMeta(DictMixin):
    """ListMeta describes metadata that synthetic resources must have, including
      lists and various status objects. A resource may have only one of {ObjectMeta,
      ListMeta}.

      **parameters**

      * **continue_** ``Optional[str]`` - continue may be set if the user set a limit on the number of items returned,
        and indicates that the server has more data available. The value is opaque and
        may be used to issue another request to the endpoint that served this list to
        retrieve the next set of available objects. Continuing a consistent list may
        not be possible if the server configuration has changed or more than a few
        minutes have passed. The resourceVersion field returned when using this
        continue value will be identical to the value in the first response, unless
        you have received this token from an error message.
      * **remainingItemCount** ``Optional[int]`` - remainingItemCount is the number of subsequent items in the list which are not
        included in this list response. If the list request contained label or field
        selectors, then the number of remaining items is unknown and the field will be
        left unset and omitted during serialization. If the list is complete (either
        because it is not chunking or because this is the last chunk), then there are
        no more remaining items and this field will be left unset and omitted during
        serialization. Servers older than v1.15 do not set this field. The intended
        use of the remainingItemCount is *estimating* the size of a collection.
        Clients should not rely on the remainingItemCount to be set or to be exact.
      * **resourceVersion** ``Optional[str]`` - String that identifies the server's internal version of this object that can
        be used by clients to determine when objects have changed. Value must be
        treated as opaque by clients and passed unmodified back to the server.
        Populated by the system. Read-only. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#concurrency-control-and-consistency
      * **selfLink** ``Optional[str]`` - Deprecated: selfLink is a legacy read-only field that is no longer populated
        by the system.
    """
    continue_: 'Optional[str]' = field(metadata={"json": "continue"}, default=None)
    remainingItemCount: 'Optional[int]' = None
    resourceVersion: 'Optional[str]' = None
    selfLink: 'Optional[str]' = None


@dataclass
class ManagedFieldsEntry(DictMixin):
    """ManagedFieldsEntry is a workflow-id, a FieldSet and the group version of the
      resource that the fieldset applies to.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the version of this resource that this field set applies
        to. The format is "group/version" just like the top-level APIVersion field. It
        is necessary to track the version of a field set because it cannot be
        automatically converted.
      * **fieldsType** ``Optional[str]`` - FieldsType is the discriminator for the different fields format and version.
        There is currently only one possible value: "FieldsV1"
      * **fieldsV1** ``Optional[FieldsV1]`` - FieldsV1 holds the first JSON version format as described in the "FieldsV1"
        type.
      * **manager** ``Optional[str]`` - Manager is an identifier of the workflow managing these fields.
      * **operation** ``Optional[str]`` - Operation is the type of operation which lead to this ManagedFieldsEntry being
        created. The only valid values for this field are 'Apply' and 'Update'.
      * **subresource** ``Optional[str]`` - Subresource is the name of the subresource used to update that object, or
        empty string if the object was updated through the main resource. The value of
        this field is used to distinguish between managers, even if they share the
        same name. For example, a status update will be distinct from a regular update
        using the same manager name. Note that the APIVersion field is not related to
        the Subresource field and it always corresponds to the version of the main
        resource.
      * **time** ``Optional[Time]`` - Time is the timestamp of when the ManagedFields entry was added. The timestamp
        will also be updated if a field is added, the manager changes any of the owned
        fields value or removes a field. The timestamp does not update when a field is
        removed from the entry because another manager took it over.
    """
    apiVersion: 'Optional[str]' = None
    fieldsType: 'Optional[str]' = None
    fieldsV1: 'Optional[FieldsV1]' = None
    manager: 'Optional[str]' = None
    operation: 'Optional[str]' = None
    subresource: 'Optional[str]' = None
    time: 'Optional[Time]' = None


MicroTime = datetime


@dataclass
class ObjectMeta(DictMixin):
    """ObjectMeta is metadata that all persisted resources must have, which includes
      all objects users must create.

      **parameters**

      * **annotations** ``Optional[dict]`` - Annotations is an unstructured key value map stored with a resource that may
        be set by external tools to store and retrieve arbitrary metadata. They are
        not queryable and should be preserved when modifying objects. More info:
        http://kubernetes.io/docs/user-guide/annotations
      * **creationTimestamp** ``Optional[Time]`` - CreationTimestamp is a timestamp representing the server time when this object
        was created. It is not guaranteed to be set in happens-before order across
        separate operations. Clients may not set this value. It is represented in
        RFC3339 form and is in UTC.
        Populated by the system. Read-only. Null for lists. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **deletionGracePeriodSeconds** ``Optional[int]`` - Number of seconds allowed for this object to gracefully terminate before it
        will be removed from the system. Only set when deletionTimestamp is also set.
        May only be shortened. Read-only.
      * **deletionTimestamp** ``Optional[Time]`` - DeletionTimestamp is RFC 3339 date and time at which this resource will be
        deleted. This field is set by the server when a graceful deletion is requested
        by the user, and is not directly settable by a client. The resource is
        expected to be deleted (no longer visible from resource lists, and not
        reachable by name) after the time in this field, once the finalizers list is
        empty. As long as the finalizers list contains items, deletion is blocked.
        Once the deletionTimestamp is set, this value may not be unset or be set
        further into the future, although it may be shortened or the resource may be
        deleted prior to this time. For example, a user may request that a pod is
        deleted in 30 seconds. The Kubelet will react by sending a graceful
        termination signal to the containers in the pod. After that 30 seconds, the
        Kubelet will send a hard termination signal (SIGKILL) to the container and
        after cleanup, remove the pod from the API. In the presence of network
        partitions, this object may still exist after this timestamp, until an
        administrator or automated process can determine the resource is fully
        terminated. If not set, graceful deletion of the object has not been
        requested.
        Populated by the system when a graceful deletion is requested. Read-only. More
        info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **finalizers** ``Optional[List[str]]`` - Must be empty before the object is deleted from the registry. Each entry is an
        identifier for the responsible component that will remove the entry from the
        list. If the deletionTimestamp of the object is non-nil, entries in this list
        can only be removed. Finalizers may be processed and removed in any order.
        Order is NOT enforced because it introduces significant risk of stuck
        finalizers. finalizers is a shared field, any actor with permission can
        reorder it. If the finalizer list is processed in order, then this can lead to
        a situation in which the component responsible for the first finalizer in the
        list is waiting for a signal (field value, external system, or other) produced
        by a component responsible for a finalizer later in the list, resulting in a
        deadlock. Without enforced ordering finalizers are free to order amongst
        themselves and are not vulnerable to ordering changes in the list.
      * **generateName** ``Optional[str]`` - GenerateName is an optional prefix, used by the server, to generate a unique
        name ONLY IF the Name field has not been provided. If this field is used, the
        name returned to the client will be different than the name passed. This value
        will also be combined with a unique suffix. The provided value has the same
        validation rules as the Name field, and may be truncated by the length of the
        suffix required to make the value unique on the server.
        If this field is specified and the generated name exists, the server will
        return a 409.
        Applied only if Name is not specified. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#idempotency
      * **generation** ``Optional[int]`` - A sequence number representing a specific generation of the desired state.
        Populated by the system. Read-only.
      * **labels** ``Optional[dict]`` - Map of string keys and values that can be used to organize and categorize
        (scope and select) objects. May match selectors of replication controllers and
        services. More info: http://kubernetes.io/docs/user-guide/labels
      * **managedFields** ``Optional[List[ManagedFieldsEntry]]`` - ManagedFields maps workflow-id and version to the set of fields that are
        managed by that workflow. This is mostly for internal housekeeping, and users
        typically shouldn't need to set or understand this field. A workflow can be
        the user's name, a controller's name, or the name of a specific apply path
        like "ci-cd". The set of fields is always in the version that the workflow
        used when modifying the object.
      * **name** ``Optional[str]`` - Name must be unique within a namespace. Is required when creating resources,
        although some resources may allow a client to request the generation of an
        appropriate name automatically. Name is primarily intended for creation
        idempotence and configuration definition. Cannot be updated. More info:
        http://kubernetes.io/docs/user-guide/identifiers#names
      * **namespace** ``Optional[str]`` - Namespace defines the space within which each name must be unique. An empty
        namespace is equivalent to the "default" namespace, but "default" is the
        canonical representation. Not all objects are required to be scoped to a
        namespace - the value of this field for those objects will be empty.
        Must be a DNS_LABEL. Cannot be updated. More info:
        http://kubernetes.io/docs/user-guide/namespaces
      * **ownerReferences** ``Optional[List[OwnerReference]]`` - List of objects depended by this object. If ALL objects in the list have been
        deleted, this object will be garbage collected. If this object is managed by a
        controller, then an entry in this list will point to this controller, with the
        controller field set to true. There cannot be more than one managing
        controller.
      * **resourceVersion** ``Optional[str]`` - An opaque value that represents the internal version of this object that can
        be used by clients to determine when objects have changed. May be used for
        optimistic concurrency, change detection, and the watch operation on a
        resource or set of resources. Clients must treat these values as opaque and
        passed unmodified back to the server. They may only be valid for a particular
        resource or set of resources.
        Populated by the system. Read-only. Value must be treated as opaque by clients
        and . More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#concurrency-control-and-consistency
      * **selfLink** ``Optional[str]`` - Deprecated: selfLink is a legacy read-only field that is no longer populated
        by the system.
      * **uid** ``Optional[str]`` - UID is the unique in time and space value for this object. It is typically
        generated by the server on successful creation of a resource and is not
        allowed to change on PUT operations.
        Populated by the system. Read-only. More info:
        http://kubernetes.io/docs/user-guide/identifiers#uids
    """
    annotations: 'Optional[dict]' = None
    creationTimestamp: 'Optional[Time]' = None
    deletionGracePeriodSeconds: 'Optional[int]' = None
    deletionTimestamp: 'Optional[Time]' = None
    finalizers: 'Optional[List[str]]' = None
    generateName: 'Optional[str]' = None
    generation: 'Optional[int]' = None
    labels: 'Optional[dict]' = None
    managedFields: 'Optional[List[ManagedFieldsEntry]]' = None
    name: 'Optional[str]' = None
    namespace: 'Optional[str]' = None
    ownerReferences: 'Optional[List[OwnerReference]]' = None
    resourceVersion: 'Optional[str]' = None
    selfLink: 'Optional[str]' = None
    uid: 'Optional[str]' = None


@dataclass
class OwnerReference(DictMixin):
    """OwnerReference contains enough information to let you identify an owning
      object. An owning object must be in the same namespace as the dependent, or be
      cluster-scoped, so there is no namespace field.

      **parameters**

      * **apiVersion** ``str`` - API version of the referent.
      * **kind** ``str`` - Kind of the referent. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **name** ``str`` - Name of the referent. More info:
        http://kubernetes.io/docs/user-guide/identifiers#names
      * **uid** ``str`` - UID of the referent. More info:
        http://kubernetes.io/docs/user-guide/identifiers#uids
      * **blockOwnerDeletion** ``Optional[bool]`` - If true, AND if the owner has the "foregroundDeletion" finalizer, then the
        owner cannot be deleted from the key-value store until this reference is
        removed. See
        https://kubernetes.io/docs/concepts/architecture/garbage-collection/#foreground-deletion
        for how the garbage collector interacts with this field and enforces the
        foreground deletion. Defaults to false. To set this field, a user needs
        "delete" permission of the owner, otherwise 422 (Unprocessable Entity) will be
        returned.
      * **controller** ``Optional[bool]`` - If true, this reference points to the managing controller.
    """
    apiVersion: 'str'
    kind: 'str'
    name: 'str'
    uid: 'str'
    blockOwnerDeletion: 'Optional[bool]' = None
    controller: 'Optional[bool]' = None


Patch = Dict


@dataclass
class Preconditions(DictMixin):
    """Preconditions must be fulfilled before an operation (update, delete, etc.) is
      carried out.

      **parameters**

      * **resourceVersion** ``Optional[str]`` - Specifies the target ResourceVersion
      * **uid** ``Optional[str]`` - Specifies the target UID.
    """
    resourceVersion: 'Optional[str]' = None
    uid: 'Optional[str]' = None


@dataclass
class ServerAddressByClientCIDR(DictMixin):
    """ServerAddressByClientCIDR helps the client to determine the server address
      that they should use, depending on the clientCIDR that they match.

      **parameters**

      * **clientCIDR** ``str`` - The CIDR with which clients can match their IP to figure out the server
        address that they should use.
      * **serverAddress** ``str`` - Address of this server, suitable for a client that matches the above CIDR.
        This can be a hostname, hostname:port, IP or IP:port.
    """
    clientCIDR: 'str'
    serverAddress: 'str'


@dataclass
class Status(DictMixin):
    """Status is a return value for calls that don't return other objects.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **code** ``Optional[int]`` - Suggested HTTP return code for this status, 0 if not set.
      * **details** ``Optional[StatusDetails]`` - Extended data associated with the reason.  Each reason may define its own
        extended details. This field is optional and the data returned is not
        guaranteed to conform to any schema except that defined by the reason type.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **message** ``Optional[str]`` - A human-readable description of the status of this operation.
      * **metadata** ``Optional[ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **reason** ``Optional[str]`` - A machine-readable description of why this operation is in the "Failure"
        status. If this value is empty there is no information available. A Reason
        clarifies an HTTP status code but does not override it.
      * **status** ``Optional[str]`` - Status of the operation. One of: "Success" or "Failure". More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    code: 'Optional[int]' = None
    details: 'Optional[StatusDetails]' = None
    kind: 'Optional[str]' = None
    message: 'Optional[str]' = None
    metadata: 'Optional[ListMeta]' = None
    reason: 'Optional[str]' = None
    status: 'Optional[str]' = None


@dataclass
class StatusCause(DictMixin):
    """StatusCause provides more information about an api.Status failure, including
      cases when multiple errors are encountered.

      **parameters**

      * **field** ``Optional[str]`` - The field of the resource that has caused this error, as named by its JSON
        serialization. May include dot and postfix notation for nested attributes.
        Arrays are zero-indexed.  Fields may appear more than once in an array of
        causes due to fields having multiple errors. Optional.
        Examples:
          "name" - the field "name" on the current resource
          "items[0].name" - the field "name" on the first array entry in "items"
      * **message** ``Optional[str]`` - A human-readable description of the cause of the error.  This field may be
        presented as-is to a reader.
      * **reason** ``Optional[str]`` - A machine-readable description of the cause of the error. If this value is
        empty there is no information available.
    """
    field: 'Optional[str]' = None
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class StatusDetails(DictMixin):
    """StatusDetails is a set of additional properties that MAY be set by the server
      to provide additional information about a response. The Reason field of a
      Status object defines what attributes will be set. Clients must ignore fields
      that do not match the defined type of each attribute, and should assume that
      any attribute may be empty, invalid, or under defined.

      **parameters**

      * **causes** ``Optional[List[StatusCause]]`` - The Causes array includes more details associated with the StatusReason
        failure. Not all StatusReasons may provide detailed causes.
      * **group** ``Optional[str]`` - The group attribute of the resource associated with the status StatusReason.
      * **kind** ``Optional[str]`` - The kind attribute of the resource associated with the status StatusReason. On
        some operations may differ from the requested resource Kind. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **name** ``Optional[str]`` - The name attribute of the resource associated with the status StatusReason
        (when there is a single name which can be described).
      * **retryAfterSeconds** ``Optional[int]`` - If specified, the time in seconds before the operation should be retried. Some
        errors may indicate the client must take an alternate action - for those
        errors this field may indicate how long to wait before taking the alternate
        action.
      * **uid** ``Optional[str]`` - UID of the resource. (when there is a single resource which can be described).
        More info: http://kubernetes.io/docs/user-guide/identifiers#uids
    """
    causes: 'Optional[List[StatusCause]]' = None
    group: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    name: 'Optional[str]' = None
    retryAfterSeconds: 'Optional[int]' = None
    uid: 'Optional[str]' = None


Time = datetime


@dataclass
class WatchEvent(DictMixin):
    """Event represents a single event to a watched resource.

      **parameters**

      * **object** ``runtime.RawExtension`` - Object is:
         * If Type is Added or Modified: the new state of the object.
         * If Type is Deleted: the state of the object immediately before deletion.
         * If Type is Error: *Status is recommended; other types may make sense
           depending on context.
      * **type** ``str`` - 
    """
    object: 'runtime.RawExtension'
    type: 'str'





# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import core_v1
from . import meta_v1


@dataclass
class ClusterCIDR(DictMixin):
    """ClusterCIDR represents a single configuration for per-Node Pod CIDR
      allocations when the MultiCIDRRangeAllocator is enabled (see the config for
      kube-controller-manager).  A cluster may have any number of ClusterCIDR
      resources, all of which will be considered when allocating a CIDR for a Node.
      A ClusterCIDR is eligible to be used for a given Node when the node selector
      matches the node in question and has free CIDRs to allocate.  In case of
      multiple matching ClusterCIDR resources, the allocator will attempt to break
      ties using internal heuristics, but any ClusterCIDR whose node selector
      matches the Node may be used.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[ClusterCIDRSpec]`` - Spec is the desired state of the ClusterCIDR. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[ClusterCIDRSpec]' = None


@dataclass
class ClusterCIDRList(DictMixin):
    """ClusterCIDRList contains a list of ClusterCIDR.

      **parameters**

      * **items** ``List[ClusterCIDR]`` - Items is the list of ClusterCIDRs.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[ClusterCIDR]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class ClusterCIDRSpec(DictMixin):
    """ClusterCIDRSpec defines the desired state of ClusterCIDR.

      **parameters**

      * **perNodeHostBits** ``int`` - PerNodeHostBits defines the number of host bits to be configured per node. A
        subnet mask determines how much of the address is used for network bits and
        host bits. For example an IPv4 address of 192.168.0.0/24, splits the address
        into 24 bits for the network portion and 8 bits for the host portion. To
        allocate 256 IPs, set this field to 8 (a /24 mask for IPv4 or a /120 for
        IPv6). Minimum value is 4 (16 IPs). This field is immutable.
      * **ipv4** ``Optional[str]`` - IPv4 defines an IPv4 IP block in CIDR notation(e.g. "10.0.0.0/8"). At least
        one of IPv4 and IPv6 must be specified. This field is immutable.
      * **ipv6** ``Optional[str]`` - IPv6 defines an IPv6 IP block in CIDR notation(e.g. "2001:db8::/64"). At least
        one of IPv4 and IPv6 must be specified. This field is immutable.
      * **nodeSelector** ``Optional[core_v1.NodeSelector]`` - NodeSelector defines which nodes the config is applicable to. An empty or nil
        NodeSelector selects all nodes. This field is immutable.
    """
    perNodeHostBits: 'int'
    ipv4: 'Optional[str]' = None
    ipv6: 'Optional[str]' = None
    nodeSelector: 'Optional[core_v1.NodeSelector]' = None





# autogenerated module
from typing import ClassVar

from ..core import resource as res
from ..models import flowcontrol_v1beta2 as m_flowcontrol_v1beta2


class FlowSchemaStatus(res.GlobalSubResource, m_flowcontrol_v1beta2.FlowSchema):
    """* **Extends**: ``models.flowcontrol_v1beta2.FlowSchema``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('flowcontrol.apiserver.k8s.io', 'v1beta2', 'FlowSchema'),
        parent=res.ResourceDef('flowcontrol.apiserver.k8s.io', 'v1beta2', 'FlowSchema'),
        plural='flowschemas',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class FlowSchema(res.GlobalResource, m_flowcontrol_v1beta2.FlowSchema):
    """* **Extends**: ``models.flowcontrol_v1beta2.FlowSchema``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``FlowSchemaStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('flowcontrol.apiserver.k8s.io', 'v1beta2', 'FlowSchema'),
        plural='flowschemas',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = FlowSchemaStatus


class PriorityLevelConfigurationStatus(res.GlobalSubResource, m_flowcontrol_v1beta2.PriorityLevelConfiguration):
    """* **Extends**: ``models.flowcontrol_v1beta2.PriorityLevelConfiguration``
       * **Type**: Global Resource
       * **Accepted client methods**: `get`, `patch`, `replace`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('flowcontrol.apiserver.k8s.io', 'v1beta2', 'PriorityLevelConfiguration'),
        parent=res.ResourceDef('flowcontrol.apiserver.k8s.io', 'v1beta2', 'PriorityLevelConfiguration'),
        plural='prioritylevelconfigurations',
        verbs=['get', 'patch', 'put'],
        action='status',
    )


class PriorityLevelConfiguration(res.GlobalResource, m_flowcontrol_v1beta2.PriorityLevelConfiguration):
    """* **Extends**: ``models.flowcontrol_v1beta2.PriorityLevelConfiguration``
       * **Type**: Global Resource
       * **Accepted client methods**: `delete`, `deletecollection`, `get`, `list`, `patch`, `create`, `replace`, `watch`

       **Subresources**:

       * **Status**: ``PriorityLevelConfigurationStatus``
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('flowcontrol.apiserver.k8s.io', 'v1beta2', 'PriorityLevelConfiguration'),
        plural='prioritylevelconfigurations',
        verbs=['delete', 'deletecollection', 'get', 'list', 'patch', 'post', 'put', 'watch'],
    )

    Status: ClassVar = PriorityLevelConfigurationStatus




# autogenerated module
from typing import List, Optional

from ._schema import dataclass, field, DictMixin

from . import meta_v1


@dataclass
class LocalSubjectAccessReview(DictMixin):
    """LocalSubjectAccessReview checks whether or not a user or group can perform an
      action in a given namespace. Having a namespace scoped resource makes it much
      easier to grant namespace scoped policy that includes permissions checking.

      **parameters**

      * **spec** ``SubjectAccessReviewSpec`` - Spec holds information about the request being evaluated.  spec.namespace must
        be equal to the namespace you made the request against.  If empty, it is
        defaulted.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **status** ``Optional[SubjectAccessReviewStatus]`` - Status is filled in by the server and indicates whether the request is allowed
        or not
    """
    spec: 'SubjectAccessReviewSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[SubjectAccessReviewStatus]' = None


@dataclass
class NonResourceAttributes(DictMixin):
    """NonResourceAttributes includes the authorization attributes available for
      non-resource requests to the Authorizer interface

      **parameters**

      * **path** ``Optional[str]`` - Path is the URL path of the request
      * **verb** ``Optional[str]`` - Verb is the standard HTTP verb
    """
    path: 'Optional[str]' = None
    verb: 'Optional[str]' = None


@dataclass
class NonResourceRule(DictMixin):
    """NonResourceRule holds information that describes a rule for the non-resource

      **parameters**

      * **verbs** ``List[str]`` - Verb is a list of kubernetes non-resource API verbs, like: get, post, put,
        delete, patch, head, options.  "*" means all.
      * **nonResourceURLs** ``Optional[List[str]]`` - NonResourceURLs is a set of partial urls that a user should have access to.
        *s are allowed, but only as the full, final step in the path.  "*" means all.
    """
    verbs: 'List[str]'
    nonResourceURLs: 'Optional[List[str]]' = None


@dataclass
class ResourceAttributes(DictMixin):
    """ResourceAttributes includes the authorization attributes available for
      resource requests to the Authorizer interface

      **parameters**

      * **group** ``Optional[str]`` - Group is the API Group of the Resource.  "*" means all.
      * **name** ``Optional[str]`` - Name is the name of the resource being requested for a "get" or deleted for a
        "delete". "" (empty) means all.
      * **namespace** ``Optional[str]`` - Namespace is the namespace of the action being requested.  Currently, there is
        no distinction between no namespace and all namespaces "" (empty) is defaulted
        for LocalSubjectAccessReviews "" (empty) is empty for cluster-scoped resources
        "" (empty) means "all" for namespace scoped resources from a
        SubjectAccessReview or SelfSubjectAccessReview
      * **resource** ``Optional[str]`` - Resource is one of the existing resource types.  "*" means all.
      * **subresource** ``Optional[str]`` - Subresource is one of the existing resource types.  "" means none.
      * **verb** ``Optional[str]`` - Verb is a kubernetes resource API verb, like: get, list, watch, create,
        update, delete, proxy.  "*" means all.
      * **version** ``Optional[str]`` - Version is the API Version of the Resource.  "*" means all.
    """
    group: 'Optional[str]' = None
    name: 'Optional[str]' = None
    namespace: 'Optional[str]' = None
    resource: 'Optional[str]' = None
    subresource: 'Optional[str]' = None
    verb: 'Optional[str]' = None
    version: 'Optional[str]' = None


@dataclass
class ResourceRule(DictMixin):
    """ResourceRule is the list of actions the subject is allowed to perform on
      resources. The list ordering isn't significant, may contain duplicates, and
      possibly be incomplete.

      **parameters**

      * **verbs** ``List[str]`` - Verb is a list of kubernetes resource API verbs, like: get, list, watch,
        create, update, delete, proxy.  "*" means all.
      * **apiGroups** ``Optional[List[str]]`` - APIGroups is the name of the APIGroup that contains the resources.  If
        multiple API groups are specified, any action requested against one of the
        enumerated resources in any API group will be allowed.  "*" means all.
      * **resourceNames** ``Optional[List[str]]`` - ResourceNames is an optional white list of names that the rule applies to.  An
        empty set means that everything is allowed.  "*" means all.
      * **resources** ``Optional[List[str]]`` - Resources is a list of resources this rule applies to.  "*" means all in the
        specified apiGroups.
         "*/foo" represents the subresource 'foo' for all resources in the specified
        apiGroups.
    """
    verbs: 'List[str]'
    apiGroups: 'Optional[List[str]]' = None
    resourceNames: 'Optional[List[str]]' = None
    resources: 'Optional[List[str]]' = None


@dataclass
class SelfSubjectAccessReview(DictMixin):
    """SelfSubjectAccessReview checks whether or the current user can perform an
      action.  Not filling in a spec.namespace means "in all namespaces".  Self is a
      special case, because users should always be able to check whether they can
      perform an action

      **parameters**

      * **spec** ``SelfSubjectAccessReviewSpec`` - Spec holds information about the request being evaluated.  user and groups
        must be empty
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **status** ``Optional[SubjectAccessReviewStatus]`` - Status is filled in by the server and indicates whether the request is allowed
        or not
    """
    spec: 'SelfSubjectAccessReviewSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[SubjectAccessReviewStatus]' = None


@dataclass
class SelfSubjectAccessReviewSpec(DictMixin):
    """SelfSubjectAccessReviewSpec is a description of the access request.  Exactly
      one of ResourceAuthorizationAttributes and NonResourceAuthorizationAttributes
      must be set

      **parameters**

      * **nonResourceAttributes** ``Optional[NonResourceAttributes]`` - NonResourceAttributes describes information for a non-resource access request
      * **resourceAttributes** ``Optional[ResourceAttributes]`` - ResourceAuthorizationAttributes describes information for a resource access
        request
    """
    nonResourceAttributes: 'Optional[NonResourceAttributes]' = None
    resourceAttributes: 'Optional[ResourceAttributes]' = None


@dataclass
class SelfSubjectRulesReview(DictMixin):
    """SelfSubjectRulesReview enumerates the set of actions the current user can
      perform within a namespace. The returned list of actions may be incomplete
      depending on the server's authorization mode, and any errors experienced
      during the evaluation. SelfSubjectRulesReview should be used by UIs to
      show/hide actions, or to quickly let an end user reason about their
      permissions. It should NOT Be used by external systems to drive authorization
      decisions as this raises confused deputy, cache lifetime/revocation, and
      correctness concerns. SubjectAccessReview, and LocalAccessReview are the
      correct way to defer authorization decisions to the API server.

      **parameters**

      * **spec** ``SelfSubjectRulesReviewSpec`` - Spec holds information about the request being evaluated.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **status** ``Optional[SubjectRulesReviewStatus]`` - Status is filled in by the server and indicates the set of actions a user can
        perform.
    """
    spec: 'SelfSubjectRulesReviewSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[SubjectRulesReviewStatus]' = None


@dataclass
class SelfSubjectRulesReviewSpec(DictMixin):
    """SelfSubjectRulesReviewSpec defines the specification for
      SelfSubjectRulesReview.

      **parameters**

      * **namespace** ``Optional[str]`` - Namespace to evaluate rules for. Required.
    """
    namespace: 'Optional[str]' = None


@dataclass
class SubjectAccessReview(DictMixin):
    """SubjectAccessReview checks whether or not a user or group can perform an
      action.

      **parameters**

      * **spec** ``SubjectAccessReviewSpec`` - Spec holds information about the request being evaluated
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **status** ``Optional[SubjectAccessReviewStatus]`` - Status is filled in by the server and indicates whether the request is allowed
        or not
    """
    spec: 'SubjectAccessReviewSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[SubjectAccessReviewStatus]' = None


@dataclass
class SubjectAccessReviewSpec(DictMixin):
    """SubjectAccessReviewSpec is a description of the access request.  Exactly one
      of ResourceAuthorizationAttributes and NonResourceAuthorizationAttributes must
      be set

      **parameters**

      * **extra** ``Optional[dict]`` - Extra corresponds to the user.Info.GetExtra() method from the authenticator.
        Since that is input to the authorizer it needs a reflection here.
      * **groups** ``Optional[List[str]]`` - Groups is the groups you're testing for.
      * **nonResourceAttributes** ``Optional[NonResourceAttributes]`` - NonResourceAttributes describes information for a non-resource access request
      * **resourceAttributes** ``Optional[ResourceAttributes]`` - ResourceAuthorizationAttributes describes information for a resource access
        request
      * **uid** ``Optional[str]`` - UID information about the requesting user.
      * **user** ``Optional[str]`` - User is the user you're testing for. If you specify "User" but not "Groups",
        then is it interpreted as "What if User were not a member of any groups
    """
    extra: 'Optional[dict]' = None
    groups: 'Optional[List[str]]' = None
    nonResourceAttributes: 'Optional[NonResourceAttributes]' = None
    resourceAttributes: 'Optional[ResourceAttributes]' = None
    uid: 'Optional[str]' = None
    user: 'Optional[str]' = None


@dataclass
class SubjectAccessReviewStatus(DictMixin):
    """SubjectAccessReviewStatus

      **parameters**

      * **allowed** ``bool`` - Allowed is required. True if the action would be allowed, false otherwise.
      * **denied** ``Optional[bool]`` - Denied is optional. True if the action would be denied, otherwise false. If
        both allowed is false and denied is false, then the authorizer has no opinion
        on whether to authorize the action. Denied may not be true if Allowed is true.
      * **evaluationError** ``Optional[str]`` - EvaluationError is an indication that some error occurred during the
        authorization check. It is entirely possible to get an error and be able to
        continue determine authorization status in spite of it. For instance, RBAC can
        be missing a role, but enough roles are still present and bound to reason
        about the request.
      * **reason** ``Optional[str]`` - Reason is optional.  It indicates why a request was allowed or denied.
    """
    allowed: 'bool'
    denied: 'Optional[bool]' = None
    evaluationError: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class SubjectRulesReviewStatus(DictMixin):
    """SubjectRulesReviewStatus contains the result of a rules check. This check can
      be incomplete depending on the set of authorizers the server is configured
      with and any errors experienced during evaluation. Because authorization rules
      are additive, if a rule appears in a list it's safe to assume the subject has
      that permission, even if that list is incomplete.

      **parameters**

      * **incomplete** ``bool`` - Incomplete is true when the rules returned by this call are incomplete. This
        is most commonly encountered when an authorizer, such as an external
        authorizer, doesn't support rules evaluation.
      * **nonResourceRules** ``List[NonResourceRule]`` - NonResourceRules is the list of actions the subject is allowed to perform on
        non-resources. The list ordering isn't significant, may contain duplicates,
        and possibly be incomplete.
      * **resourceRules** ``List[ResourceRule]`` - ResourceRules is the list of actions the subject is allowed to perform on
        resources. The list ordering isn't significant, may contain duplicates, and
        possibly be incomplete.
      * **evaluationError** ``Optional[str]`` - EvaluationError can appear in combination with Rules. It indicates an error
        occurred during rule evaluation, such as an authorizer that doesn't support
        rule evaluation, and that ResourceRules and/or NonResourceRules may be
        incomplete.
    """
    incomplete: 'bool'
    nonResourceRules: 'List[NonResourceRule]'
    resourceRules: 'List[ResourceRule]'
    evaluationError: 'Optional[str]' = None


